@@ -15,7 +15,7 @@
 """Methods to allow tket circuits to be ran on the Qulacs simulator
 """
 
-from typing import TYPE_CHECKING, Iterable, List, Optional
+from typing import TYPE_CHECKING, List, Optional, Sequence, Union, cast
 from logging import warning
 from uuid import uuid4
 import numpy as np
@@ -29,9 +29,11 @@ from pytket.backends import (
     StatusEnum,
 )
 from pytket.backends.backend import KwargTypes
+from pytket.backends.backendinfo import BackendInfo
 from pytket.backends.backendresult import BackendResult
 from pytket.backends.resulthandle import _ResultIdTuple
 from pytket.circuit import Circuit, OpType  # type: ignore
+from pytket.extensions.qulacs._metadata import __extension_version__
 from pytket.passes import (  # type: ignore
     BasePass,
     SynthesiseIBM,
@@ -50,6 +52,7 @@ from pytket.predicates import (  # type: ignore
     DefaultRegisterPredicate,
     Predicate,
 )
+from pytket.routing import Architecture  # type: ignore
 from pytket.utils.operators import QubitPauliOperator
 from pytket.utils.outcomearray import OutcomeArray
 from pytket.extensions.qulacs.qulacs_convert import tk_to_qulacs
@@ -74,10 +77,40 @@ class QulacsBackend(Backend):
     _supports_counts = True
     _supports_state = True
     _supports_expectation = True
+    _expectation_allows_nonhermitian = False
     _persistent_handles = False
+    _GATE_SET = {
+        OpType.X,
+        OpType.Y,
+        OpType.Z,
+        OpType.H,
+        OpType.S,
+        OpType.Sdg,
+        OpType.T,
+        OpType.Tdg,
+        OpType.Rx,
+        OpType.Ry,
+        OpType.Rz,
+        OpType.CX,
+        OpType.CZ,
+        OpType.SWAP,
+        OpType.U1,
+        OpType.U2,
+        OpType.U3,
+        OpType.Measure,
+        OpType.Barrier,
+    }
 
     def __init__(self) -> None:
         super().__init__()
+        self._backend_info = BackendInfo(
+            type(self).__name__,
+            None,
+            __extension_version__,
+            Architecture([]),
+            self._GATE_SET,
+        )
+
         self._sim = QuantumState
 
     @property
@@ -85,7 +118,7 @@ class QulacsBackend(Backend):
         return (str,)
 
     @property
-    def device(self) -> Optional["Device"]:
+    def backend_info(self) -> Optional["Device"]:
         return None
 
     @property
@@ -95,29 +128,7 @@ class QulacsBackend(Backend):
             NoFastFeedforwardPredicate(),
             NoMidMeasurePredicate(),
             NoSymbolsPredicate(),
-            GateSetPredicate(
-                {
-                    OpType.X,
-                    OpType.Y,
-                    OpType.Z,
-                    OpType.H,
-                    OpType.S,
-                    OpType.Sdg,
-                    OpType.T,
-                    OpType.Tdg,
-                    OpType.Rx,
-                    OpType.Ry,
-                    OpType.Rz,
-                    OpType.CX,
-                    OpType.CZ,
-                    OpType.SWAP,
-                    OpType.U1,
-                    OpType.U2,
-                    OpType.U3,
-                    OpType.Measure,
-                    OpType.Barrier,
-                }
-            ),
+            GateSetPredicate(self._GATE_SET),
             DefaultRegisterPredicate(),
         ]
 
@@ -134,17 +145,25 @@ class QulacsBackend(Backend):
 
     def process_circuits(
         self,
-        circuits: Iterable[Circuit],
-        n_shots: Optional[int] = None,
+        circuits: Sequence[Circuit],
+        n_shots: Optional[Union[int, Sequence[int]]] = None,
         valid_check: bool = True,
         **kwargs: KwargTypes,
     ) -> List[ResultHandle]:
-        circuit_list = list(circuits)
+        circuits = list(circuits)
+        if hasattr(n_shots, "__iter__"):
+            n_shots_list = list(cast(Sequence[Optional[int]], n_shots))
+            if len(n_shots_list) != len(circuits):
+                raise ValueError("The length of n_shots and circuits must match")
+        else:
+            # convert n_shots to a list
+            n_shots_list = [cast(Optional[int], n_shots)] * len(circuits)
+
         if valid_check:
-            self._check_all_circuits(circuit_list, nomeasure_warn=False)
+            self._check_all_circuits(circuits, nomeasure_warn=False)
 
         handle_list = []
-        for circuit in circuit_list:
+        for circuit, n_shots_circ in zip(circuits, n_shots_list):
             qulacs_state = self._sim(circuit.n_qubits)
             qulacs_state.set_zero_state()
             qulacs_circ = tk_to_qulacs(circuit)
@@ -153,8 +172,7 @@ class QulacsBackend(Backend):
             qubits = sorted(circuit.qubits, reverse=True)
             shots = None
             bits = None
-            if n_shots:
-
+            if n_shots_circ is not None:
                 bits2index = list(
                     (com.bits[0], qubits.index(com.qubits[0]))
                     for com in circuit
@@ -162,11 +180,11 @@ class QulacsBackend(Backend):
                 )
                 if len(bits2index) == 0:
                     bits = circuit.bits
-                    shots = OutcomeArray.from_ints([0] * n_shots, len(bits))
+                    shots = OutcomeArray.from_ints([0] * n_shots_circ, len(bits))
                 else:
                     bits, choose_indices = zip(*bits2index)
 
-                    samples = qulacs_state.sampling(n_shots)
+                    samples = qulacs_state.sampling(n_shots_circ)
                     shots = OutcomeArray.from_ints(samples, circuit.n_qubits)
                     shots = shots.choose_indices(choose_indices)
             try:
@@ -233,4 +251,5 @@ if _GPU_ENABLED:
 
         def __init__(self) -> None:
             super().__init__()
+            self._backend_info.name = type(self).__name__
             self._sim = QuantumStateGpu
